#!/usr/bin/env python

"""
poly.py
Simple algebraic manipulator for polynomials.

"""

__author__ = "Miroslav Broz (miroslav.broz@email.cz)"
__version__ = "May 16th 2020"

def plusminus(x):
  """Sign as a string"""
  if x >= 0:
    return "+"
  else:
    return "-"

def rational(x, eps = 1.0e-8, maxb = 20000):
  """Rational number as a fraction a/b"""
  if abs(x-int(x)) < eps:
    return "%g" % x

  b = 2
  while b <= maxb:
    a = x*b
    if abs(a-int(a+eps)) < eps:
      return "%d/%d " % (int(a+eps), b)
    b += 1

  return "%g" % x

class Poly(object):
  """Polynomial definition"""

  def __init__(self, a = None, n = 10, variable = "x"):
    if a != None:
       self.a = a
    else:
       self.a = n*[0.0]
    self.n = len(self.a)
    self.variable = variable

  def __str__(self):
    s = "%g" % (self.a[0])
    for i in xrange(1,len(self.a)):
      if self.a[i] != 0.0:
        s += " %s %s %s^%d" % (plusminus(self.a[i]), rational(abs(self.a[i])), self.variable, i)
    return s

  def fortran(self):
    s = "%+12.8e" % (self.a[0])
    for i in xrange(1,len(self.a)):
      if self.a[i] != 0.0:
        s += " %s %12.8e*%s(%2d)" % (plusminus(self.a[i]), abs(self.a[i]), self.variable, i)
    return s

  def evaluate(self, x):
    s = "%g" % (self.a[0])
    y = 1
    for i in xrange(1,len(self.a)):
      y *= x
      if self.a[i] != 0.0:
        s += " %s %g" % (plusminus(self.a[i]*y), abs(self.a[i]*y))
    return s


def mul(a, b):
  """Polynomial multiplication"""
  c = Poly(n = a.n+b.n-1, variable = a.variable)
  for i in xrange(0, len(a.a)):
    for j in xrange(0, len(b.a)):
      k = i+j
      c.a[k] += a.a[i]*b.a[j]
  return c

def main():
  """Test basic operations"""

  for x in [ 0.5, 0.25, 0.75, 0.125, 0.375, 0.675, 0.666666666 ]:
    print x, "	", rational(x)

  a = Poly([1, 2, 3])
  b = Poly([4, 5, 6])
  c = mul(a, b)

  print a
  print b
  print c

if __name__ == "__main__":
  main()


