#!/usr/bin/env python

"""
poly2.py
Simple algebraic manipulator for polynomials with 2 variables.

"""

__author__ = "Miroslav Broz (miroslav.broz@email.cz)"
__version__ = "May 31st 2021"

from poly import plusminus, rational

class Poly2(object):
  """Polynomial definition c x^a y^b"""

  def __init__(self, a = None, e = None, n = 1, variable = ("x", "y")):
    if a != None:
       self.a = a
       self.e = e
    else:
       self.a = n*[0.0]
       self.e = n*[(0.0, 0.0)]
    self.n = len(self.a)
    self.variable = variable

  def __str__(self):
    s = "0"
    for i in xrange(0,len(self.a)):
      if self.a[i] == 0.0:
        continue
      s += " %s %s" % (plusminus(self.a[i]), rational(abs(self.a[i])))

      for j in xrange(0,len(self.e[i])):
        if self.e[i][j] == 1.0:
          s += " %s" % (self.variable[j])
        elif self.e[i][j] != 0.0:
          s += " %s^%.0f" % (self.variable[j], self.e[i][j])

    return s

  def evaluate(self, x, y):
    s = ""
    for i in xrange(0,len(self.a)):
      z = self.a[i] * x**self.e[i][0] * y**self.e[i][1]
      s += " %s %g" % (plusminus(z), abs(z))
    return s


def mul(a, b):
  """Polynomial multiplication, incl. simplification"""
  c = Poly2(n = 0, variable = a.variable)
  for i in xrange(0, len(a.a)):
    for j in xrange(0, len(b.a)):
      exponents = (a.e[i][0]+b.e[j][0], a.e[i][1]+b.e[j][1])
      try:
        k = c.e.index(exponents)
        c.a[k] += a.a[i]*b.a[j]
      except ValueError:
        c.a.append(a.a[i]*b.a[j])
        c.e.append(exponents)
  return c

def main():
  """Test basic operations"""

  a = Poly2([1, 2, 3], [(0,0), (1,0), (2,0)])
  b = Poly2([4, 5, 6], [(0,0), (0,1), (0,2)])
  c = mul(a, b)

  print a
  print b
  print c

  a = Poly2([1, 2, 3], [(0,0), (1,0), (1,1)])
  b = Poly2([4, 5, 6], [(0,0), (0,1), (1,1)])
  c = mul(a, b)

  print ""
  print a
  print b
  print c

if __name__ == "__main__":
  main()


